/* $Header: tyrathect:Development:Perl::RCS:missing.c,v 1.2 1994/05/04 02:12:43 neeri Exp $
 *
 *    Copyright (c) 1995 Matthias Neeracher
 *
 *    You may distribute under the terms of the Perl Artistic License,
 *    as specified in the README file.
 *
 * $Log: missing.c,v $
 */

#define MAC_CONTEXT

#include "EXTERN.h"
#include "perl.h"
#include "XSUB.h"

#undef pad

#include <ICAPI.h>
#include <TFileSpec.h>
#include <Components.h>

static SV * MakeHndSV(Handle hdl)
{
	SV * res;
	char state;
	
	state = HGetState(hdl);
	HLock(hdl);
	res = GetHandleSize(hdl) ? newSVpv(*hdl, GetHandleSize(hdl)) : newSVpv("", 0);
	HSetState(hdl, state);
	
	return res;
}

MODULE = Mac::InternetConfig   PACKAGE = Mac::InternetConfig

=head1 EXTENSION

Mac::InternetConfig - Interface to Peter Lewis' and Quinns Internet Config system

=head2 Mac::InternetConfig

=cut

=item ICStart 

=item ICStart CREATOR

Call this at application initialisation. Set creator to your application creator to 
allow for future expansion of the IC system (Default is MacPerl's creator). Returns 
a connection to the IC system.

=cut

ICInstance
ICStart(creator='McPL')
	OSType	creator;
	CODE:
	if (gLastMacOSErr = ICStart(&RETVAL, creator)) {
		XSRETURN_UNDEF;
	}
	OUTPUT:
	RETVAL
	
=item ICStop INST

It is illegal to call this routine inside a ICBegin/End pair.
Call this at application termination, after which INST
is no longer valid connection to IC.

=cut

MacOSRet
ICStop(inst)
	ICInstance	inst;

=item ICGeneralFindConfigFile INST, SEARCH_PREFS, CAN_CREATE, @FOLDERS

=item ICGeneralFindConfigFile INST, SEARCH_PREFS, CAN_CREATE

=item ICGeneralFindConfigFile INST

It is illegal to call this routine inside a ICBegin/End pair.
Call to configure this connection to IC.
This routine acts as a more general replacement for
ICFindConfigFile and ICFindUserConfigFile.
Set search_prefs to 1 (default) if you want it to search the preferences folder.
Set can_create to 1 if you want it to be able to create a new config.
Set count as the number of valid elements in folders.
Set folders to a pointer to the folders to search.
Setting count to 0 and folders to nil is OK.
Searches the specified folders and then optionally the Preferences folder
in a unspecified manner.

=cut

MacOSRet
ICGeneralFindConfigFile(inst, search_prefs=1, can_create=0, ...)
	ICInstance	inst;
	Boolean		search_prefs;
	Boolean		can_create;
	PREINIT:
	int			i;
	short		count;
	FSSpec		spec;
	ICDirSpec	spex[8];
	CODE:
	count = 0;
	for (i=3; i<items; ++i)
		if (!Path2FSSpec((char *) SvPV(ST(i),na), &spec) && !FSpDown(&spec, "\p")) {
			spex[count].vRefNum = spec.vRefNum;
			spex[count].dirID	= spec.parID;
			++count;
		}
	RETVAL = ICGeneralFindConfigFile(inst, search_prefs, can_create, count, (ICDirSpecArrayPtr) spex);
	OUTPUT:
	RETVAL

=item ICChooseConfig INST

Requires IC 1.2.
It is illegal to call this routine inside a ICBegin/End pair.
Requests the user to choose a configuration, typically using some
sort of modal dialog. If the user cancels the dialog the configuration
state will be unaffected.

=cut

MacOSRet
ICChooseConfig(inst)
	ICInstance	inst;

=item ICChooseNewConfig INST

Requires IC 1.2.
It is illegal to call this routine inside a ICBegin/End pair.
Requests the user to choose a new configuration, typically using some
sort of modal dialog. If the user cancels the dialog the configuration
state will be unaffected.

=cut

MacOSRet
ICChooseNewConfig(inst)
	ICInstance	inst;

=item ICGetConfigName INST, LONGNAME
 
=item ICGetConfigName INST

Requires IC 1.2.
You must specify a configuration before calling this routine.
Returns a string that describes the current configuration at a user
level. Set longname to 1 if you want a long name, up to 255
characters, or 0 (default) if you want a short name, typically about 32
characters.
The returned string is for user display only. If you rely on the
exact format of it, you will conflict with any future IC
implementation that doesn't use explicit preference files.

=cut

Str255
ICGetConfigName(inst, longname=0)
	ICInstance	inst;
	Boolean		longname;
	CODE:
	if (gLastMacOSErr = ICGetConfigName(inst, longname, RETVAL)) {
		XSRETURN_UNDEF;
	}
	OUTPUT:
	RETVAL
	
=item ICGetConfigReference INST

Requires IC 1.2.
You must specify a configuration before calling this routine.
Returns a self-contained reference to the instance's current
configuration.

=cut

Handle
ICGetConfigReference(inst)
	ICInstance	inst;
	CODE:
	if (!(RETVAL = NewHandle(0))) {
		XSRETURN_UNDEF;
	}
	if (gLastMacOSErr = ICGetConfigReference(inst, (ICConfigRefHandle) RETVAL)) {
		DisposeHandle(RETVAL);
		XSRETURN_UNDEF;
	}
	OUTPUT:
	RETVAL

=item ICSetConfigReference INST, REF, FLAGS

=item ICSetConfigReference INST, REF

Requires IC 1.2.
It is illegal to call this routine inside a ICBegin/End pair.
Reconfigures the instance using a configuration reference that was
got using ICGetConfigReference reference. Set the
icNoUserInteraction_bit in flags if you require that this routine
not present a modal dialog. Other flag bits are reserved and should
be set to zero.

=cut

MacOSRet
ICSetConfigReference(inst, ref, flags=0)
	ICInstance	inst; 
	Handle		ref;
	long		flags;
	CODE:
	RETVAL = ICSetConfigReference(inst, (ICConfigRefHandle) ref, flags);
	OUTPUT:
	RETVAL

=item ICGetSeed INST

You do not have to specify a configuration before calling this routine.
You do not have to be inside an ICBegin/End pair to call this routine.
Returns the current seed for the IC prefs database.
This seed changes each time a non-volatile preference is changed.
You can poll this to determine if any cached preferences change.

=cut

long
ICGetSeed(inst)
	ICInstance	inst;
	CODE:
	if (gLastMacOSErr = ICGetSeed(inst, &RETVAL)) {
		XSRETURN_UNDEF;
	}
	OUTPUT:
	RETVAL

=item ICGetComponentInstance INST

Requires IC 1.2.
You do not have to specify a configuration before calling this routine.
You do not have to be inside an ICBegin/End pair to call this routine.
Returns the connection to the IC component.

=cut

ComponentInstance
ICGetComponentInstance(inst)
	ICInstance	inst;
	CODE:
	if (gLastMacOSErr = ICGetComponentInstance(inst, (Ptr *)&RETVAL)) {
		XSRETURN_UNDEF;
	}
	OUTPUT:
	RETVAL

=item ICBegin INST, PERM

You must specify a configuration before calling this routine. It is illegal to
call this routine inside a ICBegin/End pair. Starting reading or writing
multiple preferences. A call to this must be balanced by a call to ICEnd. Do
not call WaitNextEvent between these calls. The perm specifies whether you
intend to read or read/write. Only one writer is allowed per instance. Note
that this may open resource files that are not closed until you call ICEnd. 

=cut

MacOSRet
ICBegin(inst, perm)
	ICInstance	inst;
	ICPerm		perm;

=item ICGetPref INST, KEY

You must specify a configuration before calling this routine.
If you are getting or setting multiple preferences, you should place
these calls within an ICBegin/ICEnd pair.
If you call this routine outside of such a pair, it implicitly
calls ICBegin(inst, icReadOnlyPerm).
Reads the preference specified by key from the IC database to the
buffer pointed to by buf and size.
key must not be the empty string.
If called in a scalar context, return the preference. If called in a list
context, additionally returns the attributes.
Returns icPrefNotFound if there is no preference for the key.

=cut

void
ICGetPref(inst, key)
	ICInstance	inst;
	Str255		key;
	PREINIT:
	ICAttr	attr;
	Handle	pref;
	PPCODE:
	pref = NewHandle(0);
	gLastMacOSErr = ICFindPrefHandle(inst, key, &attr, pref);
	if (!gLastMacOSErr) 
		if (GIMME != G_ARRAY)
			XPUSHs(sv_2mortal(MakeHndSV(pref)));
		else {
			XPUSHs(sv_2mortal(MakeHndSV(pref)));
			XPUSHs(sv_2mortal(newSViv(attr)));
		}
	DisposeHandle(pref);

=item ICSetPref INST, KEY, VALUE
=item ICSetPref INST, KEY, VALUE, ATTR

You must specify a configuration before calling this routine.
If you are getting or setting multiple preferences, you should place
these calls within an ICBegin/ICEnd pair.
If you call this routine outside of such a pair, it implicitly
calls ICBegin(inst, icReadWritePerm).
Sets the preference specified by KEY from the IC database to the
VALUE. If attr is ICattr_no_change (the default) then the preference attributes 
are not set. Otherwise the preference attributes are set to attr.
Returns icPermErr if the previous ICBegin was passed icReadOnlyPerm.
Returns icPermErr if current attr is locked, new attr is locked.

=cut 

MacOSRet
ICSetPref(inst, key, value, attr=ICattr_no_change)
	ICInstance	inst;
	Str255		key;
	SV *		value;
	ICAttr		attr;
	PREINIT:
	STRLEN	len;
	Ptr		ptr;
	Handle	pref;
	CODE:
	ptr = SvPV(value, len);
	RETVAL = PtrToHand(ptr, &pref, len);
	if (!RETVAL) {
		RETVAL = ICSetPrefHandle(inst, key, attr, pref);
		DisposeHandle(pref);
	}
	OUTPUT:
	RETVAL

=item ICCountPref INST

You must specify a configuration before calling this routine.
You must be inside an ICBegin/End pair to call this routine.
Counts the total number of preferences.

=cut

long
ICCountPref(inst)
	ICInstance	inst;
	CODE:
	if (gLastMacOSErr = ICCountPref(inst, &RETVAL)) {
		XSRETURN_UNDEF;
	}
	OUTPUT:
	RETVAL
	
=item ICGetIndPref	INST, N

You must specify a configuration before calling this routine.
You must be inside an ICBegin/End pair to call this routine.
Returns the key of the Nth preference.
n must be positive.
Returns icPrefNotFoundErr if n is greater than the total number of preferences.

=cut

Str255
ICGetIndPref(inst, n)
	ICInstance	inst;
	long		n;
	CODE:
	if (gLastMacOSErr = ICGetIndPref(inst, n, RETVAL)) {
		XSRETURN_UNDEF;
	}
	OUTPUT:
	RETVAL

=item ICDeletePref INST, KEY

You must specify a configuration before calling this routine.
You must be inside an ICBegin/End pair to call this routine.
Deletes the preference specified by KEY.
KEY must not be the empty string.
Returns icPrefNotFound if the preference specified by key is not present.

=cut

MacOSRet
ICDeletePref(inst, key)
	ICInstance	inst;
	Str255		key;

=item ICEnd INST

You must specify a configuration before calling this routine.
You must be inside an ICBegin/End pair to call this routine.
Terminates a preference session, as started by ICBegin.
You must have called ICBegin before calling this routine.

=cut

MacOSRet
ICEnd(inst)
	ICInstance	inst;

=item ICEditPreferences	INST, KEY

Requires IC 1.1.
You must specify a configuration before calling this routine.
You do not have to be inside an ICBegin/End pair to call this routine.
Instructs IC to display the user interface associated with editing
preferences and focusing on the preference specified by key.
If key is the empty string then no preference should be focused upon.
You must have specified a configuration before calling this routine.
You do not need to call ICBegin before calling this routine.
In the current implementation this launches the IC application
(or brings it to the front) and displays the window containing
the preference specified by key.
It may have a radically different implementation in future
IC systems.

=cut

MacOSRet
ICEditPreferences(ic, key)
	ICInstance	ic;
	Str255			key;

=item ICParseURL INST, HINT, DATA, START, END

=item ICParseURL INST, HINT, DATA

Requires IC 1.1.
You must specify a configuration before calling this routine.
You do not have to be inside an ICBegin/End pair to call this routine.
Parses a URL out of the specified text and returns it in a canonical form
in a handle.
HINT indicates the default scheme for URLs of the form "name@address".
If HINT is the empty string then URLs of that form are not allowed.
DATA contains the text.
START and END should be passed in as the current selection of
the text. This selection is given in the same manner as TextEdit,
ie if START == END then there is no selection only an insertion
point. Also START  END and 0  START  length(DATA) and 0  END  length(DATA).
If START and END are omitted, the whole of DATA is assumed.
In a scalar context, returns URL. In an array context, returns URL, START, END.

=cut

void
ICParseURL(ic, hint, sv, start=-1, end=-1)
	ICInstance	ic;
	Str255			hint;
	SV *			sv;
	long			start;
	long			end;
	PREINIT:
	STRLEN	len;
	Ptr 	data;
	Handle	url;
	PPCODE:
	url = NewHandle(0);
	data = (Ptr) SvPV(sv, len);
	if (start == -1) {
		start = 0;
		end	  = len;
	} else if (end == -1) 
		end   = start;
	gLastMacOSErr = ICParseURL(ic, hint, data, len, &start, &end, url);
	if (!gLastMacOSErr) 
		if (GIMME != G_ARRAY)
			XPUSHs(sv_2mortal(MakeHndSV(url)));
		else {
			XPUSHs(sv_2mortal(MakeHndSV(url)));
			XPUSHs(sv_2mortal(newSViv(start)));
			XPUSHs(sv_2mortal(newSViv(end)));
		}
	DisposeHandle(url);

=item ICLaunchURL INST, HINT, DATA, START, END

=item ICLaunchURL INST, HINT, DATA

Requires IC 1.1.
You must specify a configuration before calling this routine.
You do not have to be inside an ICBegin/End pair to call this routine.
Parses a URL out of the specified text and feeds it off to the appropriate helper.
HINT indicates the default scheme for URLs of the form "name@address".
If HINT is the empty string then URLs of that form are not allowed.
DATA contains the text.
START and END should be passed in as the current selection of
the text. This selection is given in the same manner as TextEdit,
ie if START == END then there is no selection only an insertion
point. Also START  END and 0  START  length(DATA) and 0  END  length(DATA).
If START and END are omitted, the whole of DATA is assumed.
In a scalar context, returns URL. In an array context, returns URL, START, END.

=cut

void
ICLaunchURL(ic, hint, sv, start=-1, end=-1)
	ICInstance		ic;
	Str255			hint;
	SV *			sv;
	long			start;
	long			end;
	PREINIT:
	STRLEN	len;
	Ptr 	data;
	PPCODE:
	data = (Ptr) SvPV(sv, len);
	if (start == -1) {
		start = 0;
		end	  = len;
	} else if (end == -1) 
		end   = start;
	gLastMacOSErr = ICLaunchURL(ic, hint, data, len, &start, &end);
	if (!gLastMacOSErr) 
		if (GIMME != G_ARRAY)
			XPUSHs(sv_2mortal(newSViv(1)));
		else {
			XPUSHs(sv_2mortal(newSViv(start)));
			XPUSHs(sv_2mortal(newSViv(end)));
		}
