package Mac::SpeechRecognition;

=head1 NAME

Mac::Speech - Provide interface to Speech Recognition Manager

=head1 SYNOPSIS

	use Mac::SpeechRecognition;

=head1 DESCRIPTION

=include SpeechRecognition.xs

=head1 BUGS/LIMITATIONS

=head1 FILES

F<SpeechRecognition.pm> - Perl Interface

F<SpeechRecognition.xs> - Toolbox functions

=head1 AUTHOR(S)

Matthias Ulrich Neeracher <neeri@iis.ee.ethz.ch> Author

=cut

require Exporter;
require DynaLoader;
@ISA = (Exporter, DynaLoader);
@EXPORT = qw(
	SROpenRecognitionSystem
	SRCloseRecognitionSystem
	SRSetProperty
	SRGetProperty
	SRReleaseObject
	SRGetReference
	SRNewRecognizer
	SRStartListening
	SRStopListening
	SRSetLanguageModel
	SRGetLanguageModel
	SRContinueRecognition
	SRCancelRecognition
	SRIdle
	SRNewLanguageModel
	SRNewPath
	SRNewPhrase
	SRNewWord
	SRPutLanguageObjectIntoHandle
	SRPutLanguageObjectIntoDataFile
	SRNewLanguageObjectFromHandle
	SRNewLanguageObjectFromDataFile
	SREmptyLanguageObject
	SRChangeLanguageObject
	SRAddLanguageObject
	SRAddText
	SRRemoveLanguageObject
	SRCountItems
	SRGetIndexedItem
	SRSetIndexedItem
	SRRemoveIndexedItem
	SRDrawText
	SRDrawRecognizedText
	SRSpeakText
	SRSpeakAndDrawText
	SRStopSpeech
	SRSpeechBusy
	SRProcessBegin
	SRProcessEnd
	SRMakeSpeechObject
	gestaltSpeechRecognitionVersion
	gestaltSpeechRecognitionAttr
	gestaltDesktopSpeechRecognition
	gestaltTelephoneSpeechRecognition
	kSRNotAvailable
	kSRInternalError
	kSRComponentNotFound
	kSROutOfMemory
	kSRNotASpeechObject
	kSRBadParameter
	kSRParamOutOfRange
	kSRBadSelector
	kSRBufferTooSmall
	kSRNotARecSystem
	kSRFeedbackNotAvail
	kSRCantSetProperty
	kSRCantGetProperty
	kSRCantSetDuringRecognition
	kSRAlreadyListening
	kSRNotListeningState
	kSRModelMismatch
	kSRNoClientLanguageModel
	kSRNoPendingUtterances
	kSRRecognitionCanceled
	kSRRecognitionDone
	kSROtherRecAlreadyModal
	kSRHasNoSubItems
	kSRSubItemNotFound
	kSRLanguageModelTooBig
	kSRAlreadyReleased
	kSRAlreadyFinished
	kSRWordNotFound
	kSRNotFinishedWithRejection
	kSRExpansionTooDeep
	kSRTooManyElements
	kSRCantAdd
	kSRSndInSourceDisconnected
	kSRCantReadLanguageObject
	kSRNotImplementedYet
	uppSRCallBackProcInfo
	kSRDefaultRecognitionSystemID
	kSRFeedbackAndListeningModes
	kSRRejectedWord
	kSRCleanupOnClientExit
	kSRNoFeedbackNoListenModes
	kSRHasFeedbackHasListenModes
	kSRNoFeedbackHasListenModes
	kSRDefaultSpeechSource
	kSRLiveDesktopSpeechSource
	kSRCanned22kHzSpeechSource
	kSRNotifyRecognitionBeginning
	kSRNotifyRecognitionDone
	kAESpeechSuite
	kAESpeechDone
	kAESpeechDetected
	keySRRecognizer
	keySRSpeechResult
	keySRSpeechStatus
	typeSRRecognizer
	typeSRSpeechResult
	kSRNotificationParam
	kSRCallBackParam
	kSRSearchStatusParam
	kSRAutoFinishingParam
	kSRForegroundOnly
	kSRBlockBackground
	kSRBlockModally
	kSRWantsResultTextDrawn
	kSRWantsAutoFBGestures
	kSRSoundInVolume
	kSRReadAudioFSSpec
	kSRCancelOnSoundOut
	kSRSpeedVsAccuracyParam
	kSRUseToggleListen
	kSRUsePushToTalk
	kSRListenKeyMode
	kSRListenKeyCombo
	kSRListenKeyName
	kSRKeyWord
	kSRKeyExpected
	kSRIdleRecognizer
	kSRSearchInProgress
	kSRSearchWaitForAllClients
	kSRMustCancelSearch
	kSRPendingSearch
	kSRTEXTFormat
	kSRPhraseFormat
	kSRPathFormat
	kSRLanguageModelFormat
	kSRSpelling
	kSRLMObjType
	kSRRefCon
	kSROptional
	kSREnabled
	kSRRepeatable
	kSRRejectable
	kSRRejectionLevel
	kSRLanguageModelType
	kSRPathType
	kSRPhraseType
	kSRWordType
	kSRDefaultRejectionLevel
);

bootstrap Mac::SpeechRecognition;

sub gestaltSpeechRecognitionVersion () {     'srtb'; }
sub gestaltSpeechRecognitionAttr () {     'srta'; }
sub gestaltDesktopSpeechRecognition () {    1 << 0; }
sub gestaltTelephoneSpeechRecognition () {    1 << 1; }
sub kSRNotAvailable ()             {      -5100; }
sub kSRInternalError ()            {      -5101; }
sub kSRComponentNotFound ()        {      -5102; }
sub kSROutOfMemory ()              {      -5103; }
sub kSRNotASpeechObject ()         {      -5104; }
sub kSRBadParameter ()             {      -5105; }
sub kSRParamOutOfRange ()          {      -5106; }
sub kSRBadSelector ()              {      -5107; }
sub kSRBufferTooSmall ()           {      -5108; }
sub kSRNotARecSystem ()            {      -5109; }
sub kSRFeedbackNotAvail ()         {      -5110; }
sub kSRCantSetProperty ()          {      -5111; }
sub kSRCantGetProperty ()          {      -5112; }
sub kSRCantSetDuringRecognition () {      -5113; }
sub kSRAlreadyListening ()         {      -5114; }
sub kSRNotListeningState ()        {      -5115; }
sub kSRModelMismatch ()            {      -5116; }
sub kSRNoClientLanguageModel ()    {      -5117; }
sub kSRNoPendingUtterances ()      {      -5118; }
sub kSRRecognitionCanceled ()      {      -5119; }
sub kSRRecognitionDone ()          {      -5120; }
sub kSROtherRecAlreadyModal ()     {      -5121; }
sub kSRHasNoSubItems ()            {      -5122; }
sub kSRSubItemNotFound ()          {      -5123; }
sub kSRLanguageModelTooBig ()      {      -5124; }
sub kSRAlreadyReleased ()          {      -5125; }
sub kSRAlreadyFinished ()          {      -5126; }
sub kSRWordNotFound ()             {      -5127; }
sub kSRNotFinishedWithRejection () {      -5128; }
sub kSRExpansionTooDeep ()         {      -5129; }
sub kSRTooManyElements ()          {      -5130; }
sub kSRCantAdd ()                  {      -5131; }
sub kSRSndInSourceDisconnected ()  {      -5132; }
sub kSRCantReadLanguageObject ()   {      -5133; }
sub kSRNotImplementedYet ()        {      -5199; }
sub uppSRCallBackProcInfo ()       { kPascalStackBased; }
sub kSRDefaultRecognitionSystemID () {          0; }
sub kSRFeedbackAndListeningModes () {     'fbwn'; }
sub kSRRejectedWord ()             {     'rejq'; }
sub kSRCleanupOnClientExit ()      {     'clup'; }
sub kSRNoFeedbackNoListenModes ()  {          0; }
sub kSRHasFeedbackHasListenModes () {          1; }
sub kSRNoFeedbackHasListenModes () {          2; }
sub kSRDefaultSpeechSource ()      {          0; }
sub kSRLiveDesktopSpeechSource ()  {     'dklv'; }
sub kSRCanned22kHzSpeechSource ()  {     'ca22'; }
sub kSRNotifyRecognitionBeginning () {    1 << 0; }
sub kSRNotifyRecognitionDone ()    {    1 << 1; }
sub kAESpeechSuite ()              {     'sprc'; }
sub kAESpeechDone ()               {     'srsd'; }
sub kAESpeechDetected ()           {     'srbd'; }
sub keySRRecognizer ()             {     'krec'; }
sub keySRSpeechResult ()           {     'kspr'; }
sub keySRSpeechStatus ()           {     'ksst'; }
sub typeSRRecognizer ()            {     'trec'; }
sub typeSRSpeechResult ()          {     'tspr'; }
sub kSRNotificationParam ()        {     'noti'; }
sub kSRCallBackParam ()            {     'call'; }
sub kSRSearchStatusParam ()        {     'stat'; }
sub kSRAutoFinishingParam ()       {     'afin'; }
sub kSRForegroundOnly ()           {     'fgon'; }
sub kSRBlockBackground ()          {     'blbg'; }
sub kSRBlockModally ()             {     'blmd'; }
sub kSRWantsResultTextDrawn ()     {     'txfb'; }
sub kSRWantsAutoFBGestures ()      {     'dfbr'; }
sub kSRSoundInVolume ()            {     'volu'; }
sub kSRReadAudioFSSpec ()          {     'aurd'; }
sub kSRCancelOnSoundOut ()         {     'caso'; }
sub kSRSpeedVsAccuracyParam ()     {     'sped'; }
sub kSRUseToggleListen ()          {          0; }
sub kSRUsePushToTalk ()            {          1; }
sub kSRListenKeyMode ()            {     'lkmd'; }
sub kSRListenKeyCombo ()           {     'lkey'; }
sub kSRListenKeyName ()            {     'lnam'; }
sub kSRKeyWord ()                  {     'kwrd'; }
sub kSRKeyExpected ()              {     'kexp'; }
sub kSRIdleRecognizer ()           {    1 << 0; }
sub kSRSearchInProgress ()         {    1 << 1; }
sub kSRSearchWaitForAllClients ()  {    1 << 2; }
sub kSRMustCancelSearch ()         {    1 << 3; }
sub kSRPendingSearch ()            {    1 << 4; }
sub kSRTEXTFormat ()               {     'TEXT'; }
sub kSRPhraseFormat ()             {     'lmph'; }
sub kSRPathFormat ()               {     'lmpt'; }
sub kSRLanguageModelFormat ()      {     'lmfm'; }
sub kSRSpelling ()                 {     'spel'; }
sub kSRLMObjType ()                {     'lmtp'; }
sub kSRRefCon ()                   {     'refc'; }
sub kSROptional ()                 {     'optl'; }
sub kSREnabled ()                  {     'enbl'; }
sub kSRRepeatable ()               {     'rptb'; }
sub kSRRejectable ()               {     'rjbl'; }
sub kSRRejectionLevel ()           {     'rjct'; }
sub kSRLanguageModelType ()        {     'lmob'; }
sub kSRPathType ()                 {     'path'; }
sub kSRPhraseType ()               {     'phra'; }
sub kSRWordType ()                 {     'word'; }
sub kSRDefaultRejectionLevel ()    {         50; }

1;

__END__
